@@ -32,80 +32,6 @@ import Language.Haskell.TH.Syntax
 import Text.Read (readMaybe)
 import Type.Reflection
 
--- | 'liftType' promoted to the 'Q' monad.
---
--- @since 0.1.0.0
-liftTypeQ :: forall t. Typeable t => Q Type
-liftTypeQ = pure $ liftType @t
-
--- | Promote a 'SomeTypeRep' into a 'Type'.
---
--- @since 0.1.1.0
-typeRepToType :: SomeTypeRep -> Type
-typeRepToType (SomeTypeRep a) = go a
-  where
-    go :: forall k (a :: k). TypeRep a -> Type
-    go tr
-        | Just HRefl <- eqTypeRep (typeRep @Kind.Type) tr
-        = ConT ''Kind.Type
-        | otherwise =
-        case tr of
-            Con tyCon ->
-                mk tyCon
-            Fun trA trB ->
-                ConT ''(->) `AppT` go trA `AppT` go trB
-            App trA trB ->
-                AppT (go trA) (go trB)
-
-    mk :: TyCon -> Type
-    mk tyCon =
-        let
-            tcName =
-                tyConName tyCon
-            trySymbol =
-                case tcName of
-                    '"' : cs ->
-                        Just $ LitT (StrTyLit (zipWith const cs (drop 1 cs)))
-                    _ ->
-                        Nothing
-            tryTicked =
-                case tcName of
-                    '\'' : dcName ->
-                        let nameBase =
-                                mkOccName dcName
-
-                            flavor =
-                                NameG
-                                    DataName
-                                    (mkPkgName $ tyConPackage tyCon)
-                                    (mkModName $ tyConModule tyCon)
-                            name =
-                                Name
-                                    nameBase
-                                    flavor
-                        in
-                            Just (PromotedT name)
-                    _ ->
-                        Nothing
-            tryNat =
-                LitT . NumTyLit <$> readMaybe tcName
-            plainType =
-                let
-                    nameBase =
-                        mkOccName tcName
-                    flavor =
-                        NameG
-                            TcClsName
-                            (mkPkgName $ tyConPackage tyCon)
-                            (mkModName $ tyConModule tyCon)
-                    name =
-                        Name
-                            nameBase
-                            flavor
-                in
-                    ConT name
-        in fromMaybe plainType $ asum [tryTicked, trySymbol, tryNat]
-
 -- | Convert a type argument into a Template Haskell 'Type'.
 --
 -- Use with @TypeApplications@.
@@ -138,3 +64,137 @@ typeRepToType (SomeTypeRep a) = go a
 liftType :: forall t. Typeable t => Type
 liftType =
     typeRepToType (SomeTypeRep (typeRep @t))
+
+-- | 'liftType' promoted to the 'Q' monad.
+--
+-- @since 0.1.0.0
+liftTypeQ :: forall t. Typeable t => Q Type
+liftTypeQ = pure $ liftType @t
+
+-- | Promote a 'SomeTypeRep' into a 'Type'.
+--
+-- @since 0.1.1.0
+typeRepToType :: SomeTypeRep -> Type
+typeRepToType (SomeTypeRep a) = go a
+  where
+    go :: forall k (a :: k). TypeRep a -> Type
+    go tr
+        | Just HRefl <- eqTypeRep (typeRep @Kind.Type) tr
+        = ConT ''Kind.Type
+        | otherwise =
+        case tr of
+            Con tyCon ->
+                mk tyCon
+            Fun trA trB ->
+                ConT ''(->) `AppT` go trA `AppT` go trB
+            App trA trB ->
+                AppT (go trA) (go trB)
+
+    mk :: TyCon -> Type
+    mk tyCon =
+        let
+            tcName =
+                tyConName tyCon
+            typeOrDataName =
+                tyConToName tyCon
+            trySymbol =
+                case tcName of
+                    '"' : cs ->
+                        Just $ LitT (StrTyLit (zipWith const cs (drop 1 cs)))
+                    _ ->
+                        Nothing
+            tryTicked =
+                case typeOrDataName of
+                    PromotedDataName name ->
+                        Just (PromotedT name)
+                    _ ->
+                        Nothing
+            tryNat =
+                LitT . NumTyLit <$> readMaybe tcName
+            plainType =
+                ConT (getTypeOrDataName typeOrDataName)
+        in fromMaybe plainType $ asum [tryTicked, trySymbol, tryNat]
+
+-- | Extract the 'TypeOrDataName' from a 'TyCon'. You probably want to use
+-- 'typeToName' instead. See that function for documentation and more
+-- information.
+--
+-- @since 0.1.2.0
+tyConToName :: TyCon -> TypeOrDataName
+tyConToName tyCon =
+    let
+        tcName =
+            tyConName tyCon
+        tryTicked =
+            case tcName of
+                '\'' : dcName ->
+                    let nameBase =
+                            mkOccName dcName
+
+                        flavor =
+                            NameG
+                                DataName
+                                (mkPkgName $ tyConPackage tyCon)
+                                (mkModName $ tyConModule tyCon)
+                        name =
+                            Name
+                                nameBase
+                                flavor
+                    in
+                        Just (PromotedDataName name)
+                _ ->
+                    Nothing
+        plainType =
+            let
+                nameBase =
+                    mkOccName tcName
+                flavor =
+                    NameG
+                        TcClsName
+                        (mkPkgName $ tyConPackage tyCon)
+                        (mkModName $ tyConModule tyCon)
+                name =
+                    Name
+                        nameBase
+                        flavor
+            in
+                TypeName name
+    in fromMaybe plainType tryTicked
+
+-- | This function returns the name of the outermost type constructor.
+--
+-- >>> typeToName @Char
+-- TypeName ''Char
+-- >>> typeToName @Maybe
+-- TypeName ''Maybe
+-- >>> typeToName @(Maybe Char)
+-- TypeName ''Maybe
+-- >>> typeToName @(Int -> Char)
+-- TypeName ''(->)
+-- >>> typeToName @'False
+-- PromotedDataName 'False
+--
+-- @since 0.1.2.0
+typeToName :: forall t. Typeable t => TypeOrDataName
+typeToName = tyConToName (typeRepTyCon (typeRep @t))
+
+-- | It's possible to use a data constructor with a @DataKinds@ promotion.
+-- This disambiguates where the name comes from.
+--
+-- @since 0.1.2.0
+data TypeOrDataName
+    = TypeName Name
+    | PromotedDataName Name
+    deriving (Show, Eq)
+
+-- | Retrieve the 'Name' from a 'TypeOrDataName', forgetting how it was
+-- parsed.
+--
+-- @since 0.1.2.0
+getTypeOrDataName :: TypeOrDataName -> Name
+getTypeOrDataName d =
+    case d of
+        TypeName n ->
+            n
+        PromotedDataName n ->
+            n
